@@ -1,18 +1,39 @@
 """
 Useful utilities for data munging.
 """
+import warnings
 import pandas as pd
 import requests
-from pandas.core.indexes.multi import MultiIndex
-from pandas.core.indexes.datetimes import DatetimeIndex
-
 from dataretrieval.codes import tz
 
+
 def to_str(listlike, delimiter=','):
     """Translates list-like objects into strings.
 
-    Return:
-        List-like object as string
+    Parameters
+    ----------
+    listlike: list-like object
+        An object that is a list, or list-like
+        (e.g., ``pandas.core.series.Series``)
+    delimiter: string, optional
+        The delimiter that is placed between entries in listlike when it is
+        turned into a string. Default value is a comma.
+
+    Returns
+    -------
+    listlike: string
+        The listlike object as string separated by the delimiter
+
+    Examples
+    --------
+    .. doctest::
+
+        >>> dataretrieval.utils.to_str([1, 'a', 2])
+        '1,a,2'
+
+        >>> dataretrieval.utils.to_str([0, 10, 42], delimiter='+')
+        '0+10+42'
+
     """
     if type(listlike) == list:
         return delimiter.join([str(x) for x in listlike])
@@ -26,6 +47,7 @@ def to_str(listlike, delimiter=','):
     elif type(listlike) == str:
         return listlike
 
+
 def format_datetime(df, date_field, time_field, tz_field):
     """Creates a datetime field from separate date, time, and
     time zone fields.
@@ -34,32 +56,37 @@ def format_datetime(df, date_field, time_field, tz_field):
 
     Parameters
     ----------
-    df : DataFrame
-        DataFrame containing date, time, and timezone fields.
-
-    date_field : string
+    df: ``pandas.DataFrame``
+        A data frame containing date, time, and timezone fields.
+    date_field: string
         Name of date column in df.
-
-    time_field : string
+    time_field: string
         Name of time column in df.
-
-    tz_field : string
+    tz_field: string
         Name of time zone column in df.
 
     Returns
     -------
-    df : DataFrame
-    """
+    df: ``pandas.DataFrame``
+        The data frame with a formatted 'datetime' column
 
-    #create a datetime index from the columns in qwdata response
+    """
+    # create a datetime index from the columns in qwdata response
     df[tz_field] = df[tz_field].map(tz)
 
-    df['datetime'] = pd.to_datetime(df.pop(date_field) + ' ' +
-                                    df.pop(time_field) + ' ' +
-                                    df.pop(tz_field),
-                                    format = '%Y-%m-%d %H:%M',
+    df['datetime'] = pd.to_datetime(df[date_field] + ' ' +
+                                    df[time_field] + ' ' +
+                                    df[tz_field],
+                                    format='%Y-%m-%d %H:%M',
                                     utc=True)
 
+    # if there are any incomplete dates, warn the user
+    if any(pd.isna(df['datetime'])):
+        count = sum(pd.isna(df['datetime']) == True)
+        warnings.warn(
+            f'Warning: {count} incomplete dates found, ' +
+            'consider setting datetime_index to False.', UserWarning)
+
     return df
 
 
@@ -67,12 +94,24 @@ def format_datetime(df, date_field, time_field, tz_field):
 def update_merge(left, right, na_only=False, on=None, **kwargs):
     """Performs a combination update and merge.
 
-    Args:
-    left (DataFrame): original data
-    right (DataFrame): updated data
-    na_only (bool): if True, only update na values
+    Parameters
+    ----------
+    left: ``pandas.DataFrame``
+        Original data
+    right: ``pandas.DataFrame``
+        Updated data
+    na_only: bool
+        If True, only update na values
+
+    Returns
+    -------
+    df: ``pandas.DataFrame``
+        Updated data frame
+
+    .. todo::
+
+        add na_only parameter support
 
-    TODO: na_only
     """
     #df = left.merge(right, how='outer',
     #                left_index=True, right_index=True)
@@ -97,6 +136,8 @@ def update_merge(left, right, na_only=False, on=None, **kwargs):
 
 
 class Metadata:
+    """Custom class for metadata.
+    """
     url = None
     query_time = None
     site_info = None
@@ -111,6 +152,8 @@ class Metadata:
 
 
 def set_metadata(response):
+    """Function to initialize and set metadata from an API response.
+    """
     md = Metadata()
     md.url = response.url
     md.query_time = response.elapsed
@@ -122,15 +165,21 @@ def query(url, payload, delimiter=','):
     """Send a query.
 
     Wrapper for requests.get that handles errors, converts listed
-    query paramaters to comma separated strings, and returns response.
+    query parameters to comma separated strings, and returns response.
 
-    Args:
-        url :
-        payload : query parameters passed to requests.get
-        delimiter : delimeter to use with lists
+    Parameters
+    ----------
+    url: string
+        URL to query
+    payload: dict
+        query parameters passed to ``requests.get``
+    delimiter: string
+        delimiter to use with lists
 
-    Returns:
-        string : query response
+    Returns
+    -------
+    string: query response
+        The response from the API query ``requests.get`` function call.
     """
 
     for key, value in payload.items():
@@ -143,6 +192,20 @@ def query(url, payload, delimiter=','):
 
     if response.status_code == 400:
         raise ValueError("Bad Request, check that your parameters are correct. URL: {}".format(response.url))
+    elif response.status_code == 414:
+        _reason = response.reason
+        _example = """
+                    split_list = np.array_split(site_list, n)  # n is number of chunks to divide query into \n
+                    data_list = []  # list to store chunk results in \n
+                    # loop through chunks and make requests \n
+                    for site_list in split_list: \n
+                        data = nwis.get_record(sites=site_list, service='dv', start=start, end=end) \n
+                        data_list.append(data)  # append results to list"""
+        raise ValueError(
+            "Request URL too long. Modify your query to use fewer sites. " +
+            f"API response reason: {_reason}. Pseudo-code example of how to " +
+            f"split your query: \n {_example}"
+            )
 
     if response.text.startswith('No sites/data'):
         raise NoSitesError(response.url)
@@ -151,6 +214,8 @@ def query(url, payload, delimiter=','):
 
 
 class NoSitesError(Exception):
+    """Custom error class used when selection criteria returns no sites/data.
+    """
     def __init__(self, url):
         self.url = url
 
